#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# License: GNU General Public License v2
#
# Author: thl-cmk[at]outlook[dot]com
# URL   : https://thl-cmk.hopto.org
# Date  : 2021-07-01
#
# extended if64 to use ifNAme instead of ifDescr
#
# 2021-09-29: fixed empty interface name handling
# 2023-03-02: changed for CMK 2.1.x
# 2023-10-21: added if_fortigate to supersedes list
# 2024-06-16: modified imports for CMK 2.3

from cmk.base.plugins.agent_based.agent_based_api.v1 import (
    register,
    SNMPTree,
)

from cmk.base.plugins.agent_based.agent_based_api.v1.type_defs import (
    StringByteTable,
)

from cmk.base.plugins.agent_based.utils import interfaces
from cmk.plugins.lib import if64

_interface_displayhints = {
    'ethernet': 'eth',
    'fastethernet': 'Fa',
    'gigabitethernet': 'Gi',
    'tengigabitethernet': 'Te',
    'twentyfivegige': 'Tw',
    'fortygigabitethernet': 'Fo',
    'hundredgigabitethernet': 'Hu',
    'port-channel': 'Po',
    'tunnel': 'Tu',
    'loopback': 'Lo',
    'cellular': 'Cel',
    'vlan': 'Vlan',
    'management': 'Ma',
}


def _get_short_if_name(ifname: str) -> str:
    """
    returns a short interface name from a long interface name
    If no short name is found, the long name (ifname) will be returned
    Args:
        ifname: is the long interface name

    Returns:
        str: the short interface name as string
    """

    for ifname_prefix in _interface_displayhints.keys():
        if ifname.lower().startswith(ifname_prefix.lower()):
            ifname_short = _interface_displayhints[ifname_prefix]
            return ifname.lower().replace(ifname_prefix.lower(), ifname_short, 1)
    return ifname


def parse_if64name(string_table: StringByteTable) -> interfaces.Section:
    for interface in string_table:
        if interface[-1] != '':
            interface[1] = interface[-1]  # replace ifDescr with ifName (.1.3.6.1.2.1.31.1.1.1.1)
        interface[1] = _get_short_if_name(interface[1])  # ifDescr (.1.3.6.1.2.1.2.2.1.2)
        interface.pop()  # remove last entry from interfaces (ifName)  -> return original if64 format
    return if64.parse_if64(string_table)


register.snmp_section(
    name='if64name',
    parsed_section_name='if64',
    parse_function=parse_if64name,
    fetch=SNMPTree(
        base=if64.BASE_OID,
        oids=if64.END_OIDS + ['31.1.1.1.1']  # IF-MIB::ifName,
    ),
    detect=if64.HAS_ifHCInOctets,
    supersedes=['if64', 'if', 'statgrab_net', 'if_fortigate'],
)
